@@ -20,7 +20,7 @@
  * USA
  *
  * --------------------------------------------------------------------------
- * $Id: ActionManager.java 5343 2009-09-17 23:22:48Z alitokmen $
+ * $Id: ActionManager.java 5846 2010-01-05 13:16:36Z alitokmen $
  * --------------------------------------------------------------------------
  */
 package org.ow2.jasmine.jadort.service.implementation;
@@ -46,6 +46,7 @@ import org.ow2.jasmine.jadort.api.entities.deployment.ServerProgressState;
 import org.ow2.jasmine.jadort.api.entities.deployment.VMImageBean;
 import org.ow2.jasmine.jadort.api.entities.deployment.WorkerProgressBean;
 import org.ow2.jasmine.jadort.api.entities.deployment.WorkerProgressState;
+import org.ow2.jasmine.jadort.api.entities.deployment.ApplicationBean.ApplicationType;
 import org.ow2.jasmine.jadort.api.entities.deployment.OperationStateBean.ActionState;
 import org.ow2.jasmine.jadort.api.entities.deployment.OperationStateBean.Step;
 import org.ow2.jasmine.jadort.api.entities.topology.GroupBean;
@@ -54,6 +55,7 @@ import org.ow2.jasmine.jadort.api.entities.topology.VMBean;
 import org.ow2.jasmine.jadort.api.entities.topology.WorkerBean;
 import org.ow2.jasmine.jadort.service.action.ServerAction;
 import org.ow2.jasmine.jadort.service.action.VMMAction;
+import org.ow2.jasmine.jadort.service.action.WorkerAction;
 
 /**
  * Manages the link between server and worker states and the messages to send.
@@ -180,72 +182,154 @@ public class ActionManager extends OperationPersistenceManager {
                     throw new IllegalStateException("Unknown ServerProgressState: " + aimedServerProgressState);
                 }
             } else if (type.equals(OperationType.MIGRATE)) {
-                if (currentServerProgressState.equals(ServerProgressState.INITIAL)) {
-                    if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
-                        // next
-                        JMSSender.upload(this.operation, serverProgress, this.operation.getApplication());
-                        return;
-                    }
-                    // no previous
-                } else if (currentServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
-                    if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
-                        // next
-                        JMSSender.deploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
-                        return;
-                    } else if (aimedServerProgressState.equals(ServerProgressState.INITIAL)) {
-                        // previous
-                        JMSSender.erase(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
-                        return;
-                    }
-                } else if (currentServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
-                    if (aimedServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
-                        // next
-                        JMSSender.setAsDefault(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
-                        return;
-                    } else if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
-                        // previous
-                        JMSSender.undeploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
-                        return;
-                    }
-                } else if (currentServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
-                    if (aimedServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
-                        // next
-                        if (!serverProgress.hasOldDeploymentItem()) {
-                            throw new Exception("There's no old version or the old and new versions are the same");
-                        } else {
-                            JMSSender.undeploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                if (ApplicationType.OSGI.equals(this.operation.getApplication().getApplicationType())) {
+                    if (currentServerProgressState.equals(ServerProgressState.INITIAL)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                            // next
+                            JMSSender.upload(this.operation, serverProgress, this.operation.getApplication());
+                            return;
                         }
-                        return;
-                    } else if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
-                        // previous
-                        if (serverProgress.getOldDeploymentItem() == null) {
-                            serverProgress.appendToLog("There's no old version, ignoring step.\n"
-                                + "This implies that the new [and only] version will keep on being the default one.");
+                        // no previous
+                    } else if (currentServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
+                            if (serverProgress.getOldDeploymentItem() == null) {
+                                serverProgress.appendToLog("There's no old version, ignoring step.");
 
-                            serverProgress.setProgress(this.operation.getAimedProgressPercent());
-                            serverProgress.setActionState(ActionState.WAITING);
-                            serverProgress.setProgressState(aimedServerProgressState);
-                        } else {
-                            JMSSender.setAsDefault(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                                serverProgress.setProgress(this.operation.getAimedProgressPercent());
+                                serverProgress.setActionState(ActionState.WAITING);
+                                serverProgress.setProgressState(aimedServerProgressState);
+                            } else {
+                                // next
+                                JMSSender.undeploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.INITIAL)) {
+                            // previous
+                            JMSSender.erase(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
                         }
-                        return;
-                    }
-                } else if (currentServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
-                    if (aimedServerProgressState.equals(ServerProgressState.ERASE_OK)) {
-                        // next
-                        if (!serverProgress.hasOldDeploymentItem()) {
-                            throw new Exception("There's no old version or the old and new versions are the same");
-                        } else {
-                            JMSSender.erase(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                    } else if (currentServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                            // next
+                            JMSSender.deploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                            if (serverProgress.getOldDeploymentItem() == null) {
+                                serverProgress.appendToLog("There's no old version, ignoring step.");
+
+                                serverProgress.setProgress(this.operation.getAimedProgressPercent());
+                                serverProgress.setActionState(ActionState.WAITING);
+                                serverProgress.setProgressState(aimedServerProgressState);
+                            } else {
+                                // previous
+                                JMSSender.deploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
                         }
-                        return;
-                    } else if (aimedServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
-                        // previous
-                        JMSSender.deploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
-                        return;
+                    } else if (currentServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.ERASE_OK)) {
+                            if (serverProgress.getOldDeploymentItem() == null) {
+                                serverProgress.appendToLog("There's no old version, ignoring step.");
+
+                                serverProgress.setProgress(this.operation.getAimedProgressPercent());
+                                serverProgress.setActionState(ActionState.WAITING);
+                                serverProgress.setProgressState(aimedServerProgressState);
+                            } else {
+                                // next
+                                JMSSender.erase(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
+                            // previous
+                            JMSSender.undeploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        }
+                    } else if (currentServerProgressState.equals(ServerProgressState.ERASE_OK)) {
+                        // no next [handled by caller method]
+                        if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                            // previous
+                            if (serverProgress.getOldDeploymentItem() == null) {
+                                serverProgress.appendToLog("There's no old version, ignoring step.");
+
+                                serverProgress.setProgress(this.operation.getAimedProgressPercent());
+                                serverProgress.setActionState(ActionState.WAITING);
+                                serverProgress.setProgressState(aimedServerProgressState);
+                            } else {
+                                throw new IllegalStateException("Cannot resurrect an erased bundle!");
+                            }
+                            return;
+                        }
+                    } else {
+                        throw new IllegalStateException("Unknown ServerProgressState: " + aimedServerProgressState);
                     }
                 } else {
-                    throw new IllegalStateException("Unknown ServerProgressState: " + aimedServerProgressState);
+                    if (currentServerProgressState.equals(ServerProgressState.INITIAL)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                            // next
+                            JMSSender.upload(this.operation, serverProgress, this.operation.getApplication());
+                            return;
+                        }
+                        // no previous
+                    } else if (currentServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                            // next
+                            JMSSender.deploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.INITIAL)) {
+                            // previous
+                            JMSSender.erase(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        }
+                    } else if (currentServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
+                            // next
+                            JMSSender.setAsDefault(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.UPLOAD_OK)) {
+                            // previous
+                            JMSSender.undeploy(this.operation, serverProgress, serverProgress.getNewDeploymentItem());
+                            return;
+                        }
+                    } else if (currentServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
+                            // next
+                            if (!serverProgress.hasOldDeploymentItem()) {
+                                throw new Exception("There's no old version or the old and new versions are the same");
+                            } else {
+                                JMSSender.undeploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.DEPLOY_OK)) {
+                            // previous
+                            if (serverProgress.getOldDeploymentItem() == null) {
+                                serverProgress.appendToLog("There's no old version, ignoring step.\n"
+                                    + "This implies that the new [and only] version will keep on being the default one.");
+
+                                serverProgress.setProgress(this.operation.getAimedProgressPercent());
+                                serverProgress.setActionState(ActionState.WAITING);
+                                serverProgress.setProgressState(aimedServerProgressState);
+                            } else {
+                                JMSSender.setAsDefault(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
+                        }
+                    } else if (currentServerProgressState.equals(ServerProgressState.UNDEPLOY_OK)) {
+                        if (aimedServerProgressState.equals(ServerProgressState.ERASE_OK)) {
+                            // next
+                            if (!serverProgress.hasOldDeploymentItem()) {
+                                throw new Exception("There's no old version or the old and new versions are the same");
+                            } else {
+                                JMSSender.erase(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            }
+                            return;
+                        } else if (aimedServerProgressState.equals(ServerProgressState.SET_DEFAULT_OK)) {
+                            // previous
+                            JMSSender.deploy(this.operation, serverProgress, serverProgress.getOldDeploymentItem());
+                            return;
+                        }
+                    } else {
+                        throw new IllegalStateException("Unknown ServerProgressState: " + aimedServerProgressState);
+                    }
                 }
             } else {
                 throw new IllegalStateException("Unknown OperationType: " + type);
@@ -372,27 +456,25 @@ public class ActionManager extends OperationPersistenceManager {
     protected void fillApplications(final GroupBean groupBean) throws JadortServiceException {
         SortedMap<String, ApplicationBean> groupApplications = new TreeMap<String, ApplicationBean>();
 
-        if (groupBean.getConnected()) {
-            for (ServerBean serverBean : groupBean.getServers()) {
-                ServerAction serverAction = ServerAction.getServerAction(serverBean);
-                try {
-                    SortedMap<String, ApplicationBean> serverApplications = new TreeMap<String, ApplicationBean>();
+        for (ServerBean serverBean : groupBean.getServers()) {
+            ServerAction serverAction = ServerAction.getServerAction(serverBean);
+            try {
+                SortedMap<String, ApplicationBean> serverApplications = new TreeMap<String, ApplicationBean>();
 
-                    for (ApplicationBean applicationBean : serverAction.listOfApplications()) {
-                        groupApplications.put(applicationBean.toString(), applicationBean);
-                        serverApplications.put(applicationBean.toString(), applicationBean);
-                    }
-
-                    serverBean.setApplications(new ArrayList<ApplicationBean>(serverApplications.values()));
-                } catch (Exception e) {
-                    String message = e.getClass().getName() + ": " + e.getMessage();
-                    throw new JadortServiceException("Failed getting the applications on the server " + serverBean.getName()
-                        + ": " + message, e);
+                for (ApplicationBean applicationBean : serverAction.listOfApplications()) {
+                    groupApplications.put(applicationBean.toString(), applicationBean);
+                    serverApplications.put(applicationBean.toString(), applicationBean);
                 }
-            }
 
-            groupBean.setApplications(new ArrayList<ApplicationBean>(groupApplications.values()));
+                serverBean.setApplications(new ArrayList<ApplicationBean>(serverApplications.values()));
+            } catch (Exception e) {
+                String message = e.getClass().getName() + ": " + e.getMessage();
+                throw new JadortServiceException("Failed getting the applications on the server " + serverBean.getName()
+                    + ": " + message, e);
+            }
         }
+
+        groupBean.setApplications(new ArrayList<ApplicationBean>(groupApplications.values()));
     }
 
     /**
@@ -427,11 +509,13 @@ public class ActionManager extends OperationPersistenceManager {
      * current operation.(returns a copy since
      * operation.getTopology().getGroups() does lazy)
      * 
-     * @param fillApplications Whether to fill the applications list.
+     * @param checkConnectivity Whether to check connectivity with all servers,
+     *        workers and VMs. This will also fill the applications and VM
+     *        images lists.
      * @return The list of groups
      * @throws JadortServiceException If getting the list of applications fails.
      */
-    protected List<GroupBean> fetchGroups(final boolean fillApplications) throws JadortServiceException {
+    protected List<GroupBean> fetchGroups(final boolean checkConnectivity) throws JadortServiceException {
         if (this.operation.getTopology() != null && this.operation.getTopology().getGroups() != null) {
             // Return a copy since operation.getTopology().getGroups() does lazy
             // initialization. Remember to copy all list attributes as well!
@@ -443,7 +527,7 @@ public class ActionManager extends OperationPersistenceManager {
                 groupBeanCopy.setName(groupBean.getName());
                 groupBeanCopy.setServers(new ArrayList<ServerBean>(groupBean.getServers()));
                 groupBeanCopy.setClustered(groupBean.getClustered());
-                if (fillApplications) {
+                if (checkConnectivity) {
                     if (groupBean.getServers() != null) {
                         for (ServerBean server : groupBean.getServers()) {
                             ServerAction serverAction = ServerAction.getServerAction(server);
@@ -471,8 +555,24 @@ public class ActionManager extends OperationPersistenceManager {
                             }
                         }
                     }
+                    if (groupBean.getWorkers() != null) {
+                        for (WorkerBean worker : groupBean.getWorkers()) {
+                            WorkerAction workerAction = WorkerAction.getWorkerAction(worker);
+                            try {
+                                workerAction.getState();
+                            } catch (IOException e) {
+                                groupBeanCopy.setConnected(false);
+                            } catch (Exception e) {
+                                String message = e.getClass().getName() + ": " + e.getMessage();
+                                throw new JadortServiceException("Failed getting the state of worker "
+                                    + worker.getName() + ": " + message, e);
+                            }
+                        }
+                    }
 
-                    this.fillApplications(groupBeanCopy);
+                    if (groupBeanCopy.getConnected()) {
+                        this.fillApplications(groupBeanCopy);
+                    }
                 }
                 result.add(groupBeanCopy);
             }
